@@ -1,6 +1,6 @@
 /* gateway_commands.c
  *
- * $Id: gateway_commands.c,v 1.15 2002/03/14 15:35:08 nisse Exp $
+ * $Id: gateway_commands.c,v 1.16 2002/05/29 22:57:58 _cvs_pont Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -109,7 +109,7 @@ gateway_make_connection(struct listen_value *lv,
   /* NOTE: lv->peer is usually NULL here. */
   struct ssh_connection *connection
     = make_ssh_connection(0, /* flags */
-			  lv->peer, "gateway",
+			  lv->peer, lv->local, "gateway",
 			  make_exc_finish_read_handler(lv->fd, e, HANDLER_CONTEXT));
 
   /* Instead of calling connection_init_io. */
