@@ -2,7 +2,7 @@
  *
  * Helper program for securely executing a program as a different user.
  *
- * $Id: lsh-execuv.c,v 1.3 2002/03/15 11:27:55 nisse Exp $
+ * $Id: lsh-execuv.c,v 1.4 2002/03/25 16:42:41 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -75,22 +75,6 @@
 #include <pwd.h>
 #include <grp.h>
 
-#if 0
-#if WITH_UTMP
-# if HAVE_UTMP_H
-#  include <utmp.h>
-# endif
-
-# if HAVE_UTMPX_H
-#  include <utmpx.h>
-# endif
-#endif
-
-#if HAVE_LIBUTIL_H
-# include <libutil.h>
-#endif
-#endif
-
 static void
 usage(void)
 {
@@ -104,10 +88,6 @@ usage(void)
 	  "  -i   Call initgroups\n"
 	  "  -c   Clear the list of supplementary groups\n"
 	  "  -p   Use PATH\n"
-#if 0
-	  "  -t   Perform utmp/wtmp logging using the supplied tty name\n"
-	  "  -h   Hostname or ip address for utmp/wtmp logs\n"
-#endif
 	  "  -?   Show this help\n");
 }
 
@@ -143,55 +123,6 @@ atoid(const char *s, int *ok)
   return value;
 }
 
-#if 0
-#define CP(dst, src) (strncpy(dst, src, sizeof(dst)))
-
-#if WITH_UTMP 
-static void
-write_login_entries(const char *name, const char *tty, const char *host)
-{
-#if HAVE_UTMP_H
-  const char *prefix;
-
-  struct utmp entry;
-  memset(&entry, 0, sizeof(entry));
-
-  /* Strip matching prefix */
-  for (prefix ="/dev/" ;
-       *prefix && *tty && (*tty == *prefix);
-       tty++, prefix++)
-    ;
-
-  entry.ut_type = LOGIN_PROCESS;
-  CP(entry.ut_line, tty);
-  
-#if HAVE_STRUCT_UTMP_UT_PID
-  entry.ut_pid = getpid();
-#endif
-
-#if HAVE_STRUCT_UTMP_UT_NAME
-  CP(entry.ut_name, name);
-#endif
-
-  /* FIXME: Perform a reverse lookup.
-   * Also use ut_addr and ut_addr_v6 */
-#if HAVE_STRUCT_UTMP_UT_HOST
-  CP(entry.ut_host, host);
-#endif
-
-#if HAVE_PUTUTLINE
-  if (!pututline(&entry))
-    die_errno("pututline failed\n");
-  
-#endif
-#if HAVE_LOGWTMP
-  logwtmp(tty, name, host);
-#endif /* HAVE_LOGWTMP */
-#endif /* HAVE_UTMP_H */
-}
-#endif /* WITH_UTMP */
-#endif
-
 #if INITGROUPS_WORKAROUND
 
 /* Currently, we do this only for linux and gcc */
@@ -295,10 +226,6 @@ int main(int argc, char **argv)
   int use_path = 0;
   
   char *name = NULL;
-#if 0
-  char *tty = NULL;
-  char *host = NULL;
-#endif
   
   char *program;
   
@@ -338,15 +265,7 @@ int main(int argc, char **argv)
       case 'p':
 	use_path = 1;
 	break;
-#if 0
-      case 't':
-	tty = optarg;
-	break;
 
-      case 'h':
-	host = optarg;
-	break;
-#endif
       case -1:
 	/* Options done */
 	goto options_done;
