@@ -2,7 +2,7 @@
  *
  * client main program
  *
- * $Id: lsh.c,v 1.158 2002/01/21 00:05:05 nisse Exp $ */
+ * $Id: lsh.c,v 1.162 2002/05/14 20:58:14 _cvs_pont Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -135,22 +135,14 @@ make_options(struct exception_handler *handler,
   const char *home = getenv("HOME");
   struct randomness *r = make_user_random(home);
 
-  if (!r)
-    {
-      werror("Failed to initialize randomness generator.\n");
-      return NULL;
-    }
-  
   init_client_options(&self->super, 
 		      r,
 		      handler, exit_code);
 
-  self->algorithms
-    = make_algorithms_options(all_symmetric_algorithms());
-
+  self->algorithms = make_algorithms_options(all_symmetric_algorithms());
   self->home = home;
-  
-  self->signature_algorithms = all_signature_algorithms(r);
+
+  self->signature_algorithms = all_signature_algorithms(r); /* OK to init with NULL */
 
   self->sloppy = 0;
   self->capture = NULL;
@@ -530,17 +522,41 @@ do_lsh_lookup(struct lookup_verifier *c,
       if (!quiet_flag)
 	{
 	  /* Display fingerprint */
-	  struct lsh_string *fingerprint
-	    = hash_string(self->hash,
-			  sexp_format(subject->key, SEXP_CANONICAL, 0),
-			  1);
-			  
+
+	  struct lsh_string *spki_fingerprint = 
+	    hash_string(self->hash,
+			sexp_format(subject->key, SEXP_CANONICAL, 0),
+			1);
+
+
+	  struct lsh_string *fingerprint = 
+	    lsh_string_colonize( 
+				ssh_format( "%lfxS", 
+					    hash_string(&crypto_md5_algorithm,
+							key,
+							0)
+					    ), 
+				2, 
+				1  
+				);
+
+	  struct lsh_string *babble = 
+	    lsh_string_bubblebabble( 
+				    hash_string(&crypto_sha1_algorithm,
+						key,
+						0),
+				    1 
+				    );
+	  
 	  if (!INTERACT_YES_OR_NO
 	      (self->tty,
 	       ssh_format("Received unauthenticated key for host %lS\n"
-			  "Fingerprint: %lfxS\n"
+			  "Key details:\n"
+			  "Bubble Babble: %lfS\n"
+			  "Fingerprint:   %lfS\n"
+			  "SPKI SHA1:     %lfxS\n"
 			  "Do you trust this key? (y/n) ",
-			  self->host->ip, fingerprint), 0, 1))
+			  self->host->ip, babble, fingerprint, spki_fingerprint), 0, 1))
 	    return NULL;
 	}
 
@@ -817,6 +833,9 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	  break;
 	}
 
+      if (!self->super.random)
+	argp_failure( state, EXIT_FAILURE, 0,  "No randomness generator available.");
+
       if (self->with_dh_keyexchange < 0)
 	self->with_dh_keyexchange = !self->with_srp_keyexchange;
       
@@ -1067,11 +1086,18 @@ int main(int argc, char **argv)
   set_local_charset(CHARSET_LATIN1);
 
   options = make_options(handler, &lsh_exit_code);
+
   if (!options)
     return EXIT_FAILURE;
   
   argp_parse(&main_argp, argc, argv, ARGP_IN_ORDER, NULL, options);
 
+  if (!options->super.random)
+    {
+      werror("Failed to initialize randomness generator.\n");
+      return EXIT_FAILURE;
+    }
+
   spki = read_known_hosts(options);
   
   {
