@@ -2,7 +2,7 @@
  *
  * main proxy program.
  *
- * $Id: lsh_proxy.c,v 1.29 2001/12/12 18:37:54 nisse Exp $ */
+ * $Id: lsh_proxy.c,v 1.31 2002/03/15 09:12:37 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -121,7 +121,7 @@ const char *argp_program_bug_address = BUG_ADDRESS;
      (name lsh_proxy_options)
      (super algorithms_options)
      (vars
-       (random object randomness_with_poll)
+       (random object randomness)
        (signature_algorithms object alist)
        (style . sexp_argp_state)
        (interface . "char *")
@@ -550,7 +550,7 @@ int main(int argc, char **argv)
   
   struct alist *keys;
   
-  struct reap *reaper;
+  struct reaper *reaper;
   
   struct randomness *r;
   struct alist *algorithms_server, *algorithms_client;
