@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: read_data.c,v 1.32 2002/03/14 12:56:50 nisse Exp $ */
+ * $Id: read_data.c,v 1.35 2002/05/06 06:52:42 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -69,8 +69,21 @@ do_read_data_query(struct io_consuming_read *s)
       return 0;
     }
 
-  return MIN(self->channel->send_max_packet,
-	     self->channel->send_window_size);
+  /* There are three numbers that limit the amount of data we can read:
+   *
+   *   1 The current send_window_size.
+   *   2 The send_max_packet size for the channel.
+   *   3 (The maximum size for a complete packet SSH_MAX_PACKET)
+   *
+   * We don't enforce (3) here, but assume that if the remote end has
+   * given us a huge send_max_packet, it will also handle huge ssh
+   * packets.
+   *
+   * For channels that are forwarded via a gateway, we do need to care
+   * about (3), but that is done by the gatewaying code adjusting the
+   * send_max_packet. */
+
+  return MIN(self->channel->send_window_size, self->channel->send_max_packet);
 }
 
 
