@@ -2,7 +2,7 @@
  *
  * An implementation of SPKI certificate checking
  *
- * $Id: spki.h,v 1.24 2001/10/03 15:12:01 nisse Exp $ */
+ * $Id: spki.h,v 1.25 2002/01/21 00:05:04 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -83,12 +83,12 @@ struct sexp *
 spki_make_public_key(struct verifier *verifier);
 
 struct sexp *
-spki_hash_data(struct hash_algorithm *algorithm,
+spki_hash_data(const struct hash_algorithm *algorithm,
 	       int algorithm_name,
 	       UINT32 length, UINT8 *data);
 
 struct sexp *
-spki_hash_sexp(struct hash_algorithm *algorithm,
+spki_hash_sexp(const struct hash_algorithm *algorithm,
 	       int name,
 	       struct sexp *expr);
 
