@@ -1,6 +1,6 @@
 /* tcpforward_commands.c
  *
- * $Id: tcpforward_commands.c,v 1.35 2001/09/01 19:02:55 nisse Exp $
+ * $Id: tcpforward_commands.c,v 1.40 2001/09/28 21:54:10 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -38,30 +38,37 @@
 
 /* Forward declarations */
 
-extern struct command_2 open_direct_tcpip;
-extern struct command_2 remote_listen_command;
-extern struct command_2 open_forwarded_tcpip;
-extern struct command tcpip_start_io;
-extern struct command tcpip_connect_io;
+/* FIXME: Should be static */
+struct command_2 open_direct_tcpip;
+#define OPEN_DIRECT_TCPIP (&open_direct_tcpip.super.super)
 
-struct install_info install_direct_tcpip_handler;
-struct install_info install_forwarded_tcpip_handler;
+/* FIXME: Should be static */
+struct command_2 remote_listen_command;
+#define REMOTE_LISTEN (&remote_listen_command.super.super)
 
-/* FIXME: Should be static? */
+/* FIXME: Should be static */
+struct command_2 open_forwarded_tcpip;
+#define OPEN_FORWARDED_TCPIP (&open_forwarded_tcpip.super.super)
+
+static struct command tcpip_start_io;
+#define TCPIP_START_IO (&tcpip_start_io.super)
+
+static struct command tcpip_connect_io;
+#define TCPIP_CONNECT_IO (&tcpip_connect_io.super)
+
+static struct install_info install_direct_tcpip_handler;
+#define INSTALL_DIRECT_TCPIP (&install_direct_tcpip_handler.super.super.super)
+
+/* FIXME: Should be static */
 struct command make_direct_tcpip_handler;
+#define DIRECT_TCPIP_HANDLER (&make_direct_tcpip_handler.super)
 
-struct install_info install_tcpip_forward_handler;
+static struct install_info install_tcpip_forward_handler;
+#define INSTALL_TCPIP_FORWARD (&install_tcpip_forward_handler.super.super.super)
 
 /* FIXME: Should be static? */
 struct command make_tcpip_forward_handler;
-
-#define OPEN_DIRECT_TCPIP (&open_direct_tcpip.super.super)
-#define REMOTE_LISTEN (&remote_listen_command.super.super)
-#define TCPIP_START_IO (&tcpip_start_io.super)
-#define TCPIP_CONNECT_IO (&tcpip_connect_io.super)
-#define OPEN_FORWARDED_TCPIP (&open_forwarded_tcpip.super.super)
-#define DIRECT_TCPIP_HANDLER (&make_direct_tcpip_handler.super)
-#define INSTALL_DIRECT_TCPIP (&install_direct_tcpip_handler.super.super.super)
+#define TCPIP_FORWARD_HANDLER (&make_tcpip_forward_handler.super)
 
 static struct catch_report_collect catch_channel_open;
 #define CATCH_CHANNEL_OPEN (&catch_channel_open.super.super)
@@ -95,7 +102,7 @@ do_tcpip_connect_io(struct command *ignored UNUSED,
   COMMAND_RETURN(c, make_channel_forward(lv->fd, TCPIP_WINDOW_SIZE));
 }
 
-struct command tcpip_connect_io = STATIC_COMMAND(do_tcpip_connect_io);
+static struct command tcpip_connect_io = STATIC_COMMAND(do_tcpip_connect_io);
 
 /* Used by the party requesting tcp forwarding, i.e. when a socket is
  * already open, and we have asked the other end to forward it. Takes
@@ -117,7 +124,7 @@ do_tcpip_start_io(struct command *s UNUSED,
   COMMAND_RETURN(c, channel);
 }
 
-struct command tcpip_start_io =
+static struct command tcpip_start_io =
 { STATIC_HEADER, do_tcpip_start_io };
 
 
@@ -357,7 +364,6 @@ DEFINE_COMMAND2(remote_listen_command)
    (expr
      (name forward_local_port)
      (params
-       (backend object io_backend)
        (local object address_info)
        (target object address_info))
      (expr
@@ -370,17 +376,15 @@ DEFINE_COMMAND2(remote_listen_command)
 	       (tcpip_start_io
 	         (catch_channel_open 
 		   (open_direct_tcpip target peer) connection)))
-	     backend
 	     local)))))
 */
 
 struct command *
-make_forward_local_port(struct io_backend *backend,
-			struct address_info *local,
+make_forward_local_port(struct address_info *local,
 			struct address_info *target)
 {
   CAST_SUBTYPE(command, res,
-	       forward_local_port(backend, local, target));
+	       forward_local_port(local, target));
 
   trace("tcpforward_commands.c: forward_local_port\n");
 
@@ -407,12 +411,11 @@ make_forward_local_port(struct io_backend *backend,
 */
 
 struct command *
-make_forward_remote_port(struct io_backend *backend,
-			 struct address_info *remote,
+make_forward_remote_port(struct address_info *remote,
 			 struct address_info *target)
 {
   CAST_SUBTYPE(command, res,
-	       forward_remote_port(make_connect_port(backend, target), remote));
+	       forward_remote_port(make_connect_port(target), remote));
 
   debug("tcpforward_commands.c: forward_remote_port\n");
   
@@ -435,6 +438,7 @@ DEFINE_COMMAND(make_direct_tcpip_handler)
 		 &make_channel_open_direct_tcpip(callback)->super);
 }
 
+/* FIXME: Merge with install_tcpip_forward_handler */
 /* Takes a callback function and returns a global_request handler. */
 DEFINE_COMMAND(make_tcpip_forward_handler)
      (struct command *s UNUSED,
@@ -450,14 +454,10 @@ DEFINE_COMMAND(make_tcpip_forward_handler)
 		 &make_tcpip_forward_request(callback)->super);
 }
 
-
 /* Commands to install handlers */
-struct install_info install_direct_tcpip_handler =
+static struct install_info install_direct_tcpip_handler =
 STATIC_INSTALL_OPEN_HANDLER(ATOM_DIRECT_TCPIP);
 
-struct install_info install_forwarded_tcpip_handler =
-STATIC_INSTALL_OPEN_HANDLER(ATOM_FORWARDED_TCPIP);
-
 
 /* Server side callbacks */
 
@@ -465,44 +465,35 @@ STATIC_INSTALL_OPEN_HANDLER(ATOM_FORWARDED_TCPIP);
 /* GABA:
    (expr
      (name direct_tcpip_hook)
-     (params
-       (connect object command))
      (expr
        (lambda (connection)
          (install_direct_tcpip connection
 	   (direct_tcpip_handler (lambda (port)
-	     (tcpip_connect_io (connect connection port))))))))
+	     (tcpip_connect_io (connect_connection connection port))))))))
 */
 
 struct command *
-make_direct_tcpip_hook(struct io_backend *backend)
+make_direct_tcpip_hook(void)
 {
   CAST_SUBTYPE(command, res,
-	       direct_tcpip_hook(make_connect_connection(backend)));
-
+	       direct_tcpip_hook());
+  
   debug("tcpforward_commands.c: make_direct_tcpip_hook\n");
 
   return res;
 }
 
-struct install_info install_tcpip_forward_handler =
+static struct install_info install_tcpip_forward_handler =
 STATIC_INSTALL_GLOBAL_HANDLER(ATOM_TCPIP_FORWARD);
 
-
 /* GABA:
    (expr
      (name tcpip_forward_hook)
-     ; FIXME: Don't use globals.
-     (globals
-       (install "&install_tcpip_forward_handler.super.super.super")
-       (handler "&make_tcpip_forward_handler.super"))
-     (params
-       (backend object io_backend))
      (expr
        (lambda (connection)
          ;; Called when the ssh-connection is established
-         (install connection
-	   (handler (lambda (port)
+         (install_tcpip_forward connection
+	   (tcpip_forward_handler (lambda (port)
 	     ;; Called when the client requests remote forwarding.
 	     ;; It should return the fd associated with the port.
 	     ;; NOTE: The caller, do_tcpip_forward_request, is responsible
@@ -515,13 +506,13 @@ STATIC_INSTALL_GLOBAL_HANDLER(ATOM_TCPIP_FORWARD);
 				(tcpip_start_io
 		  		  (catch_channel_open 
 		  		    (open_forwarded_tcpip port peer) connection)))
-		  	      backend port))))))))
+		  	      port))))))))
 */
 
 struct command *
-make_tcpip_forward_hook(struct io_backend *backend)
+make_tcpip_forward_hook(void)
 {
-  CAST_SUBTYPE(command, res, tcpip_forward_hook(backend));
+  CAST_SUBTYPE(command, res, tcpip_forward_hook());
 
   debug("tcpforward_commands.c: tcpip_forward_hook\n");
   
