@@ -2,7 +2,7 @@
  *
  * User-related functions on UN*X
  *
- * $Id: unix_user.c,v 1.43 2001/12/07 16:15:38 nisse Exp $
+ * $Id: unix_user.c,v 1.48 2002/03/17 23:24:28 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -30,6 +30,7 @@
 #include "io.h"
 #include "read_file.h"
 #include "reaper.h"
+#include "server_pty.h"
 #include "werror.h"
 #include "xalloc.h"
 
@@ -81,160 +82,6 @@ struct unix_user_db;
 
 #include "unix_user.c.x"
 
-
-/* GABA:
-   (class
-     (name logout_cleanup)
-     (super exit_callback)
-     (vars
-       (process object resource)
-       (log space "struct utmp")
-       (c object exit_callback)))
-*/
-
-static void
-do_logout_cleanup(struct exit_callback *s,
-		  int signaled, int core, int value)
-{
-  CAST(logout_cleanup, self, s);
-
-  /* No need to signal the process. */
-  self->process->alive = 0;
-
-#if defined(WITH_UTMP) && defined(HAVE_LOGWTMP)
-  if (self->log)
-    logwtmp(self->log->ut_line,
-	    "",
-	    self->log->ut_host);
-#endif /* WITH_UTMP && HAVE_LOGWTMP */
-  EXIT_CALLBACK(self->c, signaled, core, value);
-}
-
-static struct exit_callback *
-make_logout_cleanup(struct resource *process,
-		    struct utmp *log,
-		    struct exit_callback *c)
-{
-  NEW(logout_cleanup, self);
-  self->super.exit = do_logout_cleanup;
-
-  self->process = process;
-  self->log = log;
-  self->c = c;
-
-  return &self->super;
-}
-
-
-#if WITH_UTMP
-static void
-lsh_strncpy(char *dst, unsigned n, struct lsh_string *s)
-{
-  unsigned length = MIN(n - 1, s->length);
-  memcpy(dst, s->data, length);
-  dst[length] = '\0';
-}
-
-/* Strips the leading "/dev/" part of s. */
-static void
-lsh_strncpy_tty(char *dst, unsigned n, struct lsh_string *s)
-{
-  /* "/dev/" is 5 characters */
-  if (s->length <= 5)
-    lsh_strncpy(dst, n, s);
-  {
-    unsigned length = MIN(n - 1, s->length - 5);
-    memcpy(dst, s->data + 5, length);
-    dst[length] = '\0';
-  }
-}
-
-#define CP(dst, src) lsh_strncpy(dst, sizeof(dst), src);
-#define CP_TTY(dst, src) lsh_strncpy_tty(dst, sizeof(dst), src);
-
-static struct utmp *
-lsh_make_utmp(struct lsh_user *user,
-	      struct address_info *peer,
-	      struct lsh_string *ttyname)
-{
-  struct utmp *log;
-  NEW_SPACE(log);
-
-#if HAVE_STRUCT_UTMP_UT_NAME
-  CP(log->ut_name, user->name);
-#elif HAVE_STRUCT_UTMP_UT_USER
-  CP(log->ut_user, user->name);
-#endif
-  
-  CP_TTY(log->ut_line, ttyname);
-
-#if HAVE_STRUCT_UTMP_UT_HOST
-  CP(log->ut_host, peer->ip);
-#endif
-  
-  return log;
-}
-#undef CP
-#undef CP_TTY
-
-#endif /* WITH_UTMP */
-
-
-/* GABA:
-   (class
-     (name process_resource)
-     (super lsh_process)
-     (vars
-       (pid . pid_t)
-       ; Signal used for killing the process.
-       (signal . int)))
-*/
-
-static void
-do_kill_process(struct resource *r)
-{
-  CAST(process_resource, self, r);
-
-  if (self->super.super.alive)
-    {
-      self->super.super.alive = 0;
-      /* NOTE: This function only makes one attempt at killing the
-       * process. An improvement would be to install a callout handler
-       * which will kill -9 the process after a delay, if it hasn't died
-       * voluntarily. */
-
-      if (kill(self->pid, self->signal) < 0)
-	{
-	  werror("do_kill_process: kill failed (errno = %i): %z\n",
-		 errno, STRERROR(errno));
-	}
-    }
-}
-
-static int
-do_signal_process(struct lsh_process *s, int signal)
-{
-  CAST(process_resource, self, s);
-  
-  return self->super.super.alive
-    && (kill(self->pid, signal) == 0);
-}
-
-
-static struct lsh_process *
-make_process_resource(pid_t pid, int signal)
-{
-  NEW(process_resource, self);
-  init_resource(&self->super.super, do_kill_process);
-  self->super.signal = do_signal_process;
-  
-  self->pid = pid;
-  self->signal = signal;
-
-  return &self->super;
-}
-
-
 /* GABA:
    (class
      (name unix_user)
@@ -698,134 +545,6 @@ do_read_file(struct lsh_user *u,
     }
 }
 
-/* Change to user's home directory. */
-
-static int
-do_chdir_home(struct lsh_user *u)
-{
-  CAST(unix_user, user, u);
-
-  if (!user->home)
-    {
-      if (chdir("/") < 0)
-	{
-	  werror("Strange: home directory was NULL, and chdir(\"/\") failed: %z\n",
-		 STRERROR(errno));
-	  return 0;
-	}
-    }
-  else if (chdir(lsh_get_cstring(user->home)) < 0)
-    {
-      werror("chdir to %S failed (using / instead): %z\n",
-	     user->home, 
-	     STRERROR(errno));
-      if (chdir("/") < 0)
-	{
-	  werror("chdir(\"/\") failed: %z\n", STRERROR(errno));
-	  return 0;
-	}
-    }
-  return 1;  
-}
-
-/* FIXME: initgroups in glibc (and possibly on other systems as well)
- * is broken, and can't handle more than 32 groups. We need a
- * workaround for that. */
-static int
-change_uid(struct unix_user *user)
-{
-  /* NOTE: Error handling is crucial here. If we do something
-   * wrong, the server will think that the user is logged in
-   * under his or her user id, while in fact the process is
-   * still running as root. */
-  if (initgroups(lsh_get_cstring(user->super.name), user->gid) < 0)
-    {
-      werror("initgroups failed: %z\n", STRERROR(errno));
-      return 0;
-    }
-  if (setgid(user->gid) < 0)
-    {
-      werror("setgid failed: %z\n", STRERROR(errno));
-      return 0;
-    }
-  if (setuid(user->super.uid) < 0)
-    {
-      werror("setuid failed: %z\n", STRERROR(errno));
-      return 0;
-    }
-  return 1;
-}
-
-/* FIXME: In child processes, between do_fork_process and do_exec_shell,
- * we're running with the user's uid, which seems dangerous.
- * It seems better to move uid handling to exec-time, and perhaps
- * to a separate program.
- *
- * The potential problem is that a user can read the process memory
- * before exec, and learn the host key or other secrets. The safest
- * way seems to be to be to exec a separate program, which changes
- * persona and then exec's the real login shell. */
-
-static int
-do_fork_process(struct lsh_user *u,
-		struct lsh_process **process,
-		struct exit_callback *c,
-		struct address_info *peer, struct lsh_string *tty)
-{
-  CAST(unix_user, user, u);
-  pid_t child;
-
-  struct utmp *log = NULL;
-  
-  /* Don't start any processes unless the user has a login shell. */
-  if (!user->shell)
-    return 0;
-
-#if WITH_UTMP
-  if (tty)
-    log = lsh_make_utmp(u, peer, tty);
-#endif
-  
-  child = fork();
-
-  switch(child)
-    {
-    case -1: 
-      werror("fork failed: %z\n", STRERROR(errno));
-      return 0;
-
-    case 0: /* Child */
-      /* FIXME: Create utmp entry as well. */
-#if defined(WITH_UTMP) && defined(HAVE_LOGWTMP)
-      if (log)
-	  /* FIXME: It should be safe to perform a blocking reverse dns lookup here,
-	   * as we have forked. */
-#if HAVE_STRUCT_UTMP_UT_NAME
-	  logwtmp(log->ut_line, log->ut_name, log->ut_host);
-#elif HAVE_STRUCT_UTMP_UT_USER
-	  logwtmp(log->ut_line, log->ut_user, log->ut_host);
-#endif
-
-#endif /* WITH_UTMP && HAVE_LOGWTMP */
-      
-      if (getuid() != user->super.uid)
-	if (!change_uid(user))
-	  {
-	    werror("Changing uid failed!\n");
-	    _exit(EXIT_FAILURE);
-	  }
-      
-      *process = NULL;
-      return 1;
-      
-    default: /* Parent */
-      *process = make_process_resource(child, SIGHUP);
-      REAP(user->ctx->reaper, child, make_logout_cleanup(&(*process)->super, log, c));
-      
-      return 1;
-    }
-}
-
 #define USE_LOGIN_DASH_CONVENTION 1
 
 static const char *
@@ -841,21 +560,43 @@ format_env_pair_c(const char *name, const char *value)
   return lsh_get_cstring(ssh_format("%lz=%lz", name, value));
 }
 
+static int
+chdir_home(struct unix_user *user)
+{
+  if (user->home)
+    {
+      if (chdir(lsh_get_cstring(user->home)) < 0)
+	werror("chdir to home directory `%S' failed (errno = %i): %z\n",
+	       user->home, errno, STRERROR(errno));
+      else
+	return 1;
+    }
+  if (chdir("/") < 0)
+    {
+      werror("chdir to `/' failed (errno = %i): %z\n",
+	     errno, STRERROR(errno));
+      return 0;
+    }
+  else
+    return 1;
+}
+
 static void
-do_exec_shell(struct lsh_user *u, int login,
-	      const char **argv,
-	      unsigned env_length,
-	      const struct env_value *env)
+exec_shell(struct unix_user *user, struct spawn_info *info)
 {
-  CAST(unix_user, user, u);
   const char **envp;
+  const char **argv;
+  const char **shell_argv;
+  const char *argv0;
+
   char *tz = getenv("TZ");
   unsigned i, j;
-  
+
+  trace("unix_user: exec_shell\n");
   assert(user->shell);
-  
+
   /* Make up an initial environment */
-  debug("do_exec_shell: Setting up environment.\n");
+  debug("exec_shell: Setting up environment.\n");
   
   /* We need place for the caller's values, 
    *
@@ -865,7 +606,7 @@ do_exec_shell(struct lsh_user *u, int login,
 
 #define MAX_ENV 6
 
-  envp = alloca(sizeof(char *) * (env_length + MAX_ENV + 1));
+  envp = alloca(sizeof(char *) * (info->env_length + MAX_ENV + 1));
 
   i = 0;
   envp[i++] = format_env_pair("SHELL", user->shell);
@@ -884,26 +625,26 @@ do_exec_shell(struct lsh_user *u, int login,
   assert(i <= MAX_ENV);
 #undef MAX_ENV
 
-  for (j = 0; j<env_length; j++)
-    envp[i++] = format_env_pair(env[j].name, env[j].value);
+  for (j = 0; j < info->env_length; j++)
+    envp[i++] = format_env_pair(info->env[j].name, info->env[j].value);
 
   envp[i] = NULL;
 
-  debug("do_exec_shell: Environment:\n");
+  debug("exec_shell: Environment:\n");
   for (i=0; envp[i]; i++)
     debug("    '%z'\n", envp[i]);
 
 #if USE_LOGIN_DASH_CONVENTION
-  if (login)
+  if (info->login)
     {
       /* Fixup argv[0], so that it starts with a dash */
       const char *p;
+      char *s;
       const char *shell = lsh_get_cstring(user->shell);
-      char *loginshell;
-      
+        
       debug("do_exec_shell: fixing up name of shell...\n");
       
-      loginshell = alloca(user->shell->length + 2);
+      s = alloca(user->shell->length + 2);
 
       /* Make sure that the shell's name begins with a -. */
       p = strrchr (shell, '/');
@@ -912,20 +653,222 @@ do_exec_shell(struct lsh_user *u, int login,
       else
 	p ++;
 
-      loginshell[0] = '-';
-      strncpy (loginshell + 1, p, user->shell->length);
-      
-      argv[0] = loginshell;
+      s[0] = '-';
+      strncpy (s + 1, p, user->shell->length);
+      argv0 = s;
     }
   else
 #endif /* USE_LOGIN_DASH_CONVENTION */
-    argv[0] = lsh_get_cstring(user->shell);
+    argv0 = lsh_get_cstring(user->shell);
 
-  debug("do_exec_shell: argv[0] = '%z'.\n", argv[0]);
+  debug("exec_shell: argv0 = '%z'.\n", argv0);
 
-  /* FIXME: Is there a better way? The execve prototype uses char *
-   * const argv, and similarly for envp. */
-  execve(lsh_get_cstring(user->shell), (char **) argv, (char **) envp);
+  /* Build argument list for lsh-execuv. We need place for
+   *
+   * lsh-execuv -u uid -g gid -n name -i -- $SHELL argv0 <user args> NULL
+   */
+#define MAX_ARG 11
+#define NUMBER(x) lsh_get_cstring(ssh_format("%di", (x)))
+  
+  argv = alloca(sizeof(char *) * (MAX_ARG + info->argc + 1));
+  i = 0;
+  argv[i++] = "lsh-execuv";
+  argv[i++] = "-u";
+  trace("exec_shell: After -u\n");
+  argv[i++] = NUMBER(user->super.uid);
+  argv[i++] = "-g";
+  trace("exec_shell: After -g\n");
+  argv[i++] = NUMBER(user->gid);
+  argv[i++] = "-n";
+  argv[i++] = lsh_get_cstring(user->super.name);
+  argv[i++] = "-i";
+  trace("exec_shell: After -i\n");
+  argv[i++] = "--";
+  argv[i++] = lsh_get_cstring(user->shell);
+  shell_argv = argv + i;
+  
+  argv[i++] = argv0;
+
+  assert(i <= MAX_ARG);
+#undef MAX_ARG
+#undef NUMBER
+  for (j = 0; j<info->argc; j++)
+    argv[i++] = info->argv[j];
+  argv[i++] = NULL;
+  
+  debug("exec_shell: Argument list:\n");
+  for (i=0; argv[i]; i++)
+    debug("    '%z'\n", argv[i]);
+
+  /* NOTE: The execve prototype uses char * const argv, and similarly
+   * for envp, which seems broken. */
+  
+  trace("exec_shell: before exec\n");
+
+  /* Use lsh-execuv only if we need to change our uid. */
+  if (user->super.uid == getuid())
+    execve(lsh_get_cstring(user->shell), (char **) shell_argv, (char **) envp);
+  else
+    execve(PREFIX "/sbin/lsh-execuv", (char **) argv, (char **) envp);
+
+  werror("unix_user: exec failed (errno = %i): %z\n",
+	 errno, STRERROR(errno));
+  _exit(EXIT_FAILURE);
+}
+
+static void
+safe_close(int fd)
+{
+  if (fd != -1 && close(fd) < 0)
+    werror("close failed (errno = %i): %z\n",
+	   errno, STRERROR(errno));
+}
+
+static struct lsh_process *
+do_spawn(struct lsh_user *u,
+	 struct spawn_info *info,
+	 struct exit_callback *c)
+{
+  CAST(unix_user, user, u);
+  /* Pipe used for syncronization. */
+  int sync[2];
+  pid_t child;
+
+  if (!lsh_make_pipe(sync))
+    {
+      werror("do_spawn: Failed to create syncronization pipe.\n");
+      return NULL;
+    }
+  
+  child = fork();
+  if (child < 0)
+    {
+      werror("fork failed: %z\n", STRERROR(errno));
+      safe_close(sync[0]); safe_close(sync[1]);
+
+      safe_close(info->in[0]);  safe_close(info->in[1]);
+      safe_close(info->out[0]); safe_close(info->out[1]);
+      safe_close(info->err[0]); safe_close(info->err[1]);
+
+      return NULL;
+    }
+  else if (child)
+    {
+      /* Parent */
+      struct lsh_process *process;
+      char dummy;
+      int res;
+      
+      trace("do_spawn: parent process\n");
+
+      /* Close the child's fd:s, except ones that are -1 */
+      safe_close(info->in[0]);
+      safe_close(info->out[1]);
+      safe_close(info->err[1]);
+
+      safe_close(sync[1]);
+
+      /* On Solaris, reading the master side of the pty before the
+       * child has opened the slave side of it results in EINVAL. We
+       * can't have that, so we'll wait until the child has opened the
+       * tty, after which it should close its end of the
+       * syncronization pipe, and our read will return 0.
+       *
+       * We need the syncronizatino only if we're actually using a
+       * pty, but for simplicity, we do it every time. */
+      
+      do
+	res = read(sync[0], &dummy, 1);
+      while (res < 0 && errno == EINTR);
+
+      safe_close(sync[0]);
+
+      trace("do_spawn: parent after sync\n");
+      
+      process = unix_process_setup(child, &user->super, &c,
+				   info->peer,
+				   info->pty ? info->pty->tty_name : NULL);
+
+      trace("do_spawn: parent after process setup\n");
+
+      REAP(user->ctx->reaper, child, c);
+      return process;
+    }
+  else
+    { /* Child */
+      int tty = -1;
+
+      trace("do_spawn: child process\n");
+      if (!chdir_home(user))
+	_exit(EXIT_FAILURE);
+
+      trace("do_spawn: child after chdir\n");
+      
+#if WITH_PTY_SUPPORT
+      if (info->pty)
+	{
+	  debug("lshd: unix_user.c: Opening slave tty...\n");
+	  if ( (tty = pty_open_slave(info->pty)) < 0)
+	    {
+	      debug("lshd: unix_user.c: "
+		    "Opening slave tty... Failed!\n");
+	      werror("lshd: Can't open controlling tty for child!\n");
+	      _exit(EXIT_FAILURE);
+	    }
+	  else
+	    debug("lshd: unix_user.c: Opening slave tty... Ok.\n");
+	}
+#endif /* WITH_PTY_SUPPORT */
+
+      trace("do_spawn: child after pty\n");
+      
+      /* Now any tty processing is done, so notify our parent by
+       * closing the syncronization pipe. */
+      
+      safe_close(sync[0]); safe_close(sync[1]);
+
+#define DUP_FD_OR_TTY(src, dst) dup2((src) >= 0 ? (src) : tty, dst)
+
+      if (DUP_FD_OR_TTY(info->in[0], STDIN_FILENO) < 0)
+	{
+	  werror("Can't dup stdin!\n");
+	  _exit(EXIT_FAILURE);
+	}
+
+      if (DUP_FD_OR_TTY(info->out[1], STDOUT_FILENO) < 0)
+	{
+	  werror("Can't dup stdout!\n");
+	  _exit(EXIT_FAILURE);
+	}
+
+      trace("do_spawn: child before stderr dup\n");
+      if (!dup_error_stream())
+	{
+	  werror("unix_user.c: Failed to dup old stderr. Bye.\n");
+	  set_error_ignore();
+	}
+
+      if (DUP_FD_OR_TTY(info->err[1], STDERR_FILENO) < 0)
+	{
+	  werror("Can't dup stderr!\n");
+	  _exit(EXIT_FAILURE);
+	}
+#undef DUP_FD_OR_TTY
+      
+      trace("do_spawn: child after stderr dup\n");
+
+      /* Close all the fd:s, except ones that are -1 */
+      safe_close(info->in[0]);
+      safe_close(info->in[1]);
+      safe_close(info->out[0]);
+      safe_close(info->out[1]);
+      safe_close(info->err[0]);
+      safe_close(info->err[1]);
+      safe_close(tty);
+      
+      exec_shell(user, info);
+      _exit(EXIT_FAILURE);
+    }
 }
 
 static struct lsh_user *
@@ -944,9 +887,7 @@ make_unix_user(struct lsh_string *name,
   user->super.verify_password = do_verify_password;
   user->super.file_exists = do_file_exists;
   user->super.read_file = do_read_file;
-  user->super.chdir_home = do_chdir_home;
-  user->super.fork_process = do_fork_process;
-  user->super.exec_shell = do_exec_shell;
+  user->super.spawn = do_spawn;
   
   user->super.uid = uid;
   user->gid = gid;
@@ -969,7 +910,7 @@ make_unix_user(struct lsh_string *name,
      (name unix_user_db)
      (super user_db)
      (vars
-       (reaper object reap)
+       (reaper object reaper)
 
        ; A program to use for verifying passwords.
        (pw_helper . "const char *")
@@ -1086,7 +1027,8 @@ do_lookup_user(struct user_db *s,
 	/* Override the passwd database */
 	shell = self->login_shell;
       else
-	shell = passwd->pw_shell;
+	/* Default login shell is /bin/sh */
+	shell = passwd->pw_shell ? passwd->pw_shell : "/bin/sh";
       
       return make_unix_user(free ? name : lsh_string_dup(name), 
 			    passwd->pw_uid, passwd->pw_gid,
@@ -1104,7 +1046,7 @@ do_lookup_user(struct user_db *s,
 }
 
 struct user_db *
-make_unix_user_db(struct reap *reaper,
+make_unix_user_db(struct reaper *reaper,
 		  const char *pw_helper, const char *login_shell,
 		  int allow_root)
 {
