@@ -1,6 +1,6 @@
 /* userauth.h
  *
- * $Id: userauth.h,v 1.31 2001/10/02 22:38:00 nisse Exp $
+ * $Id: userauth.h,v 1.35 2002/03/16 22:10:47 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -34,6 +34,7 @@
 
 /* Forward declaration */
 struct env_value;
+struct spawn_info;
 
 #define GABA_DECLARE
 #include "userauth.h.x"
@@ -68,6 +69,35 @@ make_userauth_special_exception(struct lsh_string *reply,
 
 #define SIGNAL_PROCESS(p, s) ((p)->signal((p), (s)))
 
+struct spawn_info
+{
+  /* For logging */
+  struct address_info *peer;
+  /* If a pty should be allocated */
+  struct pty_info *pty;
+
+  /* Is it a login session? */
+  int login ;
+  
+  /* {in|out|err}[0] is for reading,
+   * {in|out|err}[1] for writing. */
+
+  /* Negative values for the child fd:s means that the slave tty should
+   * be used. */
+  int in[2]; int out[2]; int err[2];
+
+  /* These are the arguments to the shell, not including
+   * the traditional argv[0]. */
+  unsigned argc;
+  const char **argv;
+  
+  /* Currently, this environment is used also for the
+   * execution of the lsh-execuv program, so dangerous
+   * variables must not be set. */
+  unsigned env_length;
+  const struct env_value *env;
+};
+
 /* GABA:
    (class
      (name lsh_user)
@@ -93,45 +123,27 @@ make_userauth_special_exception(struct lsh_string *reply,
        (read_file method "const struct exception *"
                   "const char *name" "int secret"
 		  "UINT32 limit"
-		  ; Get's the contents of the file
+		  ; Gets the contents of the file
 		  "struct abstract_write *c")
 		  
-       ; chdir to the user's home directory
-       (chdir_home method int)
-
-       ; Fork a user process.
-
-       ; FIXME: For non-UN*X-systems, we need a method
-       ; that combines fork() and exec(). It would have lots of
-       ; input arguments, perhaps a tag list, and return
-       ; an object containing the stdin file objects and perhaps some
-       ; other info.
-
-       ; Perhaps POSIX process_spawn is suitable?
-
-       ; This function also returns the pid.
-
-       ; The tty argument is for utmp/wtmp logging
-       (fork_process method int "struct lsh_process **child"
-                     "struct exit_callback *c"
-                     "struct address_info * peer" "struct lsh_string *tty")
-
-       ; This modifies the argv vector, in particular its first
-       ; element. So the vector must have at least two elements,
-       ; (argv[0], NULL). 
-
-       (exec_shell method void "int login"
-                   "const char **argv"
-		   "unsigned env_length"
-		   "const struct env_value *env")))
+       ; Spawns a user process executing the user's login shell.
+       ; Also closes the appropriate fd:s in info. '
+       (spawn method "struct lsh_process *"
+              "struct spawn_info *info"
+              "struct exit_callback *c")))
 */
 
 #define USER_VERIFY_PASSWORD(u, p, c, e) ((u)->verify_password((u), (p), (c), (e)))
 #define USER_FILE_EXISTS(u, n, f) ((u)->file_exists((u), (n), (f)))
 #define USER_READ_FILE(u, n, s, l, c) ((u)->read_file((u), (n), (s), (l), (c)))
-#define USER_CHDIR_HOME(u) ((u)->chdir_home((u)))
-#define USER_FORK(u, c, e, p, t) \
-  ((u)->fork_process((u), (c), (e), (p), (t)))
-#define USER_EXEC(u, m, a, l, e) ((u)->exec_shell((u), (m), (a), (l), (e)))
+#define USER_SPAWN(u, i, c) ((u)->spawn((u), (i), (c)))
+
+/* This prototype doesn't really belong here. */
+struct lsh_process *
+unix_process_setup(pid_t pid,
+		   struct lsh_user *user,
+		   struct exit_callback **c,
+		   struct address_info *peer,
+		   struct lsh_string *tty);
 
 #endif /* LSH_USERAUTH_H_INCLUDED */
